<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

// Requires core WP List Table class...
if ( ! class_exists( 'WP_List_Table' ) ) {
    require_once( ABSPATH . 'wp-admin/includes/class.wp-list-table.php' );
}

/**
 * List table for viewing all API activity...
 *
 * @author      Ryan Bayne
 * @category    Admin
 * @package     TwitchPress/Views
 * @version     1.0
 */
class TwitchPress_ListTable_APIActivity extends WP_List_Table {

    /**
     * Max items.
     *
     * @var int
     */
    protected $max_items;

    public $items = array();
    
    /**
     * Constructor.
     */
    public function __construct() {

        parent::__construct( array(
            'singular'  => __( 'Entry', 'twitchpress' ),
            'plural'    => __( 'Entries', 'twitchpress' ),
            'ajax'      => false
        ) );
        
        // Apply default items to the $items object.
        $this->default_items();
    }   
    
    /**
    * Setup default items. 
    * 
    * This is not required and was only implemented for demonstration purposes. 
    * 
    * @version 1.3
    */
    public function default_items() {
        global $wpdb;
        $entry_counter = 0;// Acts as temporary ID for data that does not have one. 
        
        $records = twitchpress_db_selectorderby( $wpdb->twitchpress_activity, null, 'callid' );
        
        if( !isset( $records ) || !is_array( $records ) ) { $records = array(); } 

        // Loop on individual trace entries. 
        foreach( $records as $key => $row ) {

            ++$entry_counter;
            
            // Create new array entry and get it's key...
            $this->items[]['entry_number'] = $entry_counter; 
            end( $this->items);
            $key = key( $this->items );
            
            $this->items[$key]['entryid']   = $row->entryid;
            $this->items[$key]['callid']    = $row->callid;
            $this->items[$key]['service']   = $row->service;
            $this->items[$key]['type']      = $row->type;
            $this->items[$key]['outcome']   = $row->outcome;
            $this->items[$key]['timestamp'] = $row->timestamp;
            $this->items[$key]['wpuserid']  = $row->wpuserid;
        }   
        
        $this->items = array_reverse( $this->items );                                                   
    }
    
    /**
     * No items found text.
     */
    public function no_items() {
        _e( 'No items found.', 'twitchpress' );
    }

    public function display_tablenav( $position ) {
        if ( $position != 'top' ) { parent::display_tablenav( $position ); }
    }

    /**
     * Output the tabled report
     */
    public function output_result() {
        $this->prepare_items();
        echo '<div id="poststuff" class="twitchpress-tablelist-wide">';
        $this->display();
        echo '</div>';
    }

    /**
     * Get column value.
     *
     * @param mixed $item
     * @param string $column_name   
     * 
     * @version 1.0
     */
    public function column_default( $item, $column_name ) {
        switch( $column_name ) {
            case 'timestamp' :
                $time_passed = human_time_diff( strtotime( $item['timestamp'] ), time() );
                echo sprintf( __( '%s ago', 'twitchpress' ), $time_passed );
            break;
            case 'callid' :
                echo '<pre>'; print_r( $item['callid'] ); echo '</pre>';
            break;
            
            case 'service' :
                echo '<pre>'; print_r( $item['service'] ); echo '</pre>';
            break;            
        
            case 'type' :
                echo '<pre>'; print_r( $item['type'] ); echo '</pre>';
            break;            
            
            case 'outcome' :
                echo '<pre>'; print_r( $item['outcome'] ); echo '</pre>';
            break;                                    
            
            case 'wpuserid' :
                echo '<pre>'; print_r( $item['wpuserid'] ); echo '</pre>';
            break;                                    
        }
    }

    /**
     * Get columns.
     *
     * @return array
     */
    public function get_columns() {
        $columns = array(
            'timestamp' => __( 'Date/Time', 'twitchpress' ),
            'callid'    => __( 'Call ID', 'twitchpress' ),
            'service'   => __( 'Service', 'twitchpress' ),
            'type'      => __( 'Type', 'twitchpress' ),
            'outcome'   => __( 'Outcome', 'twitchpress' ),
            'wpuserid'  => __( 'WP User ID', 'twitchpress' ),
        );
        return $columns;
    }

    /**
     * Prepare customer list items.
     */
    public function prepare_items() {

        $this->_column_headers = array( $this->get_columns(), array(), $this->get_sortable_columns() );
        $current_page          = absint( $this->get_pagenum() );
        $per_page              = apply_filters( 'twitchpress_listtable_apiactivity_items_per_page', 20 );

        $this->get_items( $current_page, $per_page );

        /**
         * Pagination.
         */
        $this->set_pagination_args( array(
            'total_items' => $this->max_items,
            'per_page'    => $per_page,
            'total_pages' => ceil( $this->max_items / $per_page )
        ) );
    }
}
            
